"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a, _b, _c;
Object.defineProperty(exports, "__esModule", { value: true });
exports.manifestSchema = exports.serializeAndSaveManifest = exports.validateAndReadManifest = void 0;
var node_path_1 = __importDefault(require("node:path"));
var node_fs_1 = __importDefault(require("node:fs"));
var yaml_1 = __importDefault(require("yaml"));
var joi_1 = __importDefault(require("joi"));
var constants_1 = require("./constants");
var utils_1 = require("./utils");
var validateAndReadManifest = function (logger, program, projectPath) {
    logger.ok('Validating Mamoru manifest');
    var manifestPath = getManifest(logger, program, projectPath);
    var manifestSrc = node_fs_1.default.readFileSync(manifestPath, 'utf-8');
    logger.verbose("Content of \"".concat(manifestPath, "\" is"), manifestSrc);
    var manifest = yaml_1.default.parse(manifestSrc);
    validateManifestContentCLI(logger, program, manifest);
    return manifest;
};
exports.validateAndReadManifest = validateAndReadManifest;
var serializeAndSaveManifest = function (logger, manifest, buildPath) {
    var compiledManifestPath = node_path_1.default.join(buildPath, 'manifest.yml');
    var compiledManifest = yaml_1.default.stringify(manifest);
    logger.verbose("Saving compiled manifest to \"".concat(compiledManifestPath, "\""));
    node_fs_1.default.writeFileSync(compiledManifestPath, compiledManifest);
};
exports.serializeAndSaveManifest = serializeAndSaveManifest;
function getManifest(logger, program, projectPath) {
    var ymlPath = node_path_1.default.join(projectPath, 'manifest.yml');
    var yamlPath = node_path_1.default.join(projectPath, 'manifest.yaml');
    logger.verbose("Checking if \"".concat(ymlPath, "\" exists"));
    if (node_fs_1.default.existsSync(ymlPath)) {
        logger.verbose("\"".concat(ymlPath, "\" exists"));
        return ymlPath;
    }
    logger.verbose("Checking if \"".concat(yamlPath, "\" exists"));
    if (node_fs_1.default.existsSync(yamlPath)) {
        logger.verbose("\"".concat(yamlPath, "\" exists"));
        return yamlPath;
    }
    program.error(' manifest not found.');
}
var manifestParameterSchema = joi_1.default.object().keys({
    type: joi_1.default.any().valid('STRING', 'NUMBER', 'BOOLEAN').required(),
    key: joi_1.default.string().required().pattern(constants_1.ONLY_ALPHA_NUMERIC),
    title: joi_1.default.string().required(),
    description: joi_1.default.string().optional(),
    defaultValue: joi_1.default.string().optional(),
    requiredFor: joi_1.default
        .array()
        .items((_a = joi_1.default.string()).valid.apply(_a, (0, utils_1.getAvailableChains)()))
        .optional(),
    hiddenFor: joi_1.default
        .array()
        .items((_b = joi_1.default.string()).valid.apply(_b, (0, utils_1.getAvailableChains)()))
        .optional(),
});
/**
 * export for testing
 */
exports.manifestSchema = joi_1.default.object().keys({
    version: joi_1.default.any().valid('0.0.1').required(),
    type: joi_1.default.any().valid('wasm', 'sql').required(),
    description: joi_1.default.string().optional(),
    subscribable: joi_1.default.boolean().optional(),
    name: joi_1.default.string().required(),
    chains: joi_1.default
        .array()
        .items((_c = joi_1.default.string()).valid.apply(_c, (0, utils_1.getAvailableChains)()))
        .min(1),
    tags: joi_1.default.array().items(joi_1.default.string()).optional(),
    logoUrl: joi_1.default
        .string()
        .uri({ scheme: ['http', 'https'] })
        .optional(),
    parameters: joi_1.default.array().items(manifestParameterSchema).optional(),
});
function validateManifestContentCLI(logger, program, manifest) {
    logger.verbose("Validating manifest");
    var error = exports.manifestSchema.validate(manifest).error;
    if (error) {
        var formatted = (0, utils_1.formatJoiError)(error);
        program.error(formatted);
    }
}
//# sourceMappingURL=manifest.js.map