"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getProto = exports.getData = void 0;
/* eslint-disable no-case-declarations */
var protobufjs_1 = __importDefault(require("protobufjs"));
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
var Reader = protobufjs_1.default.Reader;
// indent by count
var indent = function (count) { return Array(count).join('  '); };
// is a number a float?
var isFloat = function (n) { return Number(n) === n && n % 1 !== 0; };
// turn a message into a proto-representation
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
var handleMessage = function (msg, m, level) {
    if (m === void 0) { m = 'Root'; }
    if (level === void 0) { level = 1; }
    var seen = [];
    var repeated = [];
    var lines = msg
        .map(function (field) {
        var n = Object.keys(field).pop();
        var t = Array.isArray(field[n]) ? 'array' : typeof field[n];
        switch (t) {
            case 'object': // it's a buffer
                return "".concat(indent(level + 1), "bytes field").concat(n, " = ").concat(n, "; // could be a repeated-value, string, bytes, or malformed sub-message");
            case 'string':
                return "".concat(indent(level + 1), "string field").concat(n, " = ").concat(n, "; // could be a repeated-value, string, bytes, or malformed sub-message");
            case 'number':
                return isFloat(field[n])
                    ? "".concat(indent(level + 1), "float field").concat(n, " = ").concat(n, "; // could be a fixed64, sfixed64, double, fixed32, sfixed32, or float")
                    : "".concat(indent(level + 1), "int32 field").concat(n, " = ").concat(n, "; // could be a int32, int64, uint32, bool, enum, etc, or even a float of some kind");
            case 'array': // sub-message
                if (seen.indexOf(n) === -1) {
                    seen.push(n);
                    return "\n".concat(handleMessage(field[n], n, level + 1), "\n").concat(indent(level + 1), "\n").concat(indent(level + 1), "Message").concat(n, " subMessage").concat(n, " = ").concat(n, ";");
                }
                else {
                    repeated.push(n);
                }
        }
    })
        .filter(function (l) { return l; });
    var repeatHandled = [];
    repeated.forEach(function (num) {
        lines.forEach(function (l, i) {
            if (l.indexOf("subMessage".concat(num)) !== -1 &&
                repeatHandled.indexOf(num) === -1) {
                lines[i] = l.replace("Message".concat(num, " subMessage").concat(num), "repeated Message".concat(num, " subMessage").concat(num));
                repeatHandled.push(num);
            }
        });
    });
    return "".concat(indent(level), "message Message").concat(m, " {\n").concat(lines.join('\n'), "\n").concat(indent(level), "}");
};
/**
 * Turn a protobuf into a data-object
 *
 * @param      {Buffer}   buffer     The proto in a binary buffer
 * @param      {string}   stringMode How to handle strings that aren't sub-messages: "auto" - guess based on chars, "string" - always a string, "binary" - always a buffer
 * @return     {object[]}            Info about the protobuf
 */
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
var getData = function (buffer, stringMode) {
    if (stringMode === void 0) { stringMode = 'auto'; }
    var reader = Reader.create(buffer);
    var out = [];
    var _loop_1 = function () {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        var tag = reader.uint64();
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        var id = tag >>> 3;
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        var wireType = tag & 7;
        switch (wireType) {
            case 0: // int32, int64, uint32, bool, enum, etc
                out.push((_a = {}, _a[id] = reader.uint32(), _a));
                break;
            case 1: // fixed64, sfixed64, double
                out.push((_b = {}, _b[id] = reader.fixed64(), _b));
                break;
            case 2: // string, bytes, sub-message
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                var bytes = reader.bytes();
                try {
                    var innerMessage = (0, exports.getData)(bytes, stringMode);
                    out.push((_c = {}, _c[id] = innerMessage, _c));
                }
                catch (e) {
                    if (stringMode === 'binary') {
                        out.push((_d = {}, _d[id] = bytes, _d));
                    }
                    else if (stringMode === 'string') {
                        out.push((_e = {}, _e[id] = bytes.toString(), _e));
                    }
                    else {
                        // search buffer for extended chars
                        var hasExtended_1 = false;
                        bytes.forEach(function (b) {
                            if (b < 32) {
                                hasExtended_1 = true;
                            }
                        });
                        if (hasExtended_1) {
                            out.push((_f = {}, _f[id] = bytes, _f));
                        }
                        else {
                            out.push((_g = {}, _g[id] = bytes.toString(), _g));
                        }
                    }
                }
                break;
            // IGNORE start_group
            // IGNORE end_group
            case 5: // fixed32, sfixed32, float
                out.push((_h = {}, _h[id] = reader.float(), _h));
                break;
            default:
                reader.skipType(wireType);
        }
    };
    while (reader.pos < reader.len) {
        _loop_1();
    }
    return out;
};
exports.getData = getData;
/**
 * Gets the proto-definition string from a binary protobuf message
 *
 * @param      {Buffer}  buffer  The buffer
 * @param      {string}   stringMode How to handle strings that aren't sub-messages: "auto" - guess based on chars, "string" - always a string, "binary" - always a buffer
 * @return     {string}  The proto
 */
var getProto = function (buffer, stringMode) {
    if (stringMode === void 0) { stringMode = 'auto'; }
    var data = (0, exports.getData)(buffer, stringMode);
    var out = 'syntax = "proto3";\n\n';
    out += handleMessage(data);
    return out;
};
exports.getProto = getProto;
//# sourceMappingURL=rawproto.js.map